# -*- coding: utf-8 -*-
# Copyright © 2005 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that creates a trigger for striim's
window manager.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2005 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE

class Trigger(GObject):
	"""
	This trigger initializes striim's window manager. It also provides
	global triggers to manipulate striim's window.
	"""

	__gsignals__ = {
		"destroy": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, player):
		"""
		Initialize the trigger.

		@param self: Reference to the Trigger instance.
		@type self: An Trigger object.

		@param player: Reference to the player.
		@type player: An Player object.
		"""
		GObject.__init__(self)
		self.__init_attributes(player)
		self.__create_triggers()
		self.__signal_id_1 = self.__show_trigger.connect("activate", self.__show_window_cb)
		self.__signal_id_2 = self.__hide_trigger.connect("activate", self.__hide_window_cb)
		self.__signal_id_3 = self.__toggle_trigger.connect("activate", self.__toggle_window_cb)
		self.__signal_id_4 = self.connect("destroy", self.__destroy_cb)
		self.__signal_id_5 = self.__close_trigger.connect("activate", self.__close_window_cb)

	def __init_attributes(self, player):
		"""
		Initialize the trigger's attributes.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param player: Reference to the player.
		@type player: An Player object.
		"""
		self.__player = player
		# Initialize striim's window manager.
		from Window import Window
		self.__manager = Window(self, player)
		self.__show_trigger = self.__hide_trigger = None
		self.__close_trigger = self.__toggle_trigger = None
		self.__signal_id_1 = self.__signal_id_2 = None
		self.__signal_id_3 = self.__signal_id_4 = None
		self.__signal_id_5 = None
		return

	def __create_triggers(self):
		"""
		Create the trigger.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		from Striim.Trigger import Trigger

		# Trigger to hide the player window.
		self.__hide_trigger = Trigger("hide_player", "Escape")
		self.__player.add_trigger(self.__hide_trigger)

		# Trigger to show the player window.
		self.__show_trigger = Trigger("show_player")
		self.__player.add_trigger(self.__show_trigger)

		# Trigger to toggle player's visibility.
		self.__toggle_trigger = Trigger("toggle_player_visibility")
		self.__player.add_trigger(self.__toggle_trigger)

		# Trigger to close the player.
		self.__close_trigger = Trigger("close_player", "ctrl - w")
		self.__player.add_trigger(self.__close_trigger)
		return

	def __show_window_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param trigger: An object that shows striim's window.
		@type trigger: A Trigger object.
		"""
		self.__manager.show_window()
		return

	def __hide_window_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param trigger: An object that hides striim's window.
		@type trigger: A Trigger object.
		"""
		self.__manager.hide_window()
		return

	def __toggle_window_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param trigger: An object that shows/hides striim's window.
		@type trigger: A Trigger object.
		"""
		self.__manager.toggle_window()
		return

	def __close_window_cb(self, trigger):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.

		@param trigger: An object that closes striim.
		@type trigger: A Trigger object.
		"""
		self.__manager.close_window()
		self.__player.emit("quit")
		self.__manager.emit("destroy")
		return

	def __destroy_cb(self, trigger):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the Trigger instance.
		@type self: An Trigger object.

		@param trigger: Reference to the Trigger instance.
		@type trigger: An Trigger object.
		"""
		triggers = [self.__show_trigger, self.__hide_trigger,
					self.__toggle_trigger, self.__close_trigger]
		self.__player.remove_triggers(triggers)
		from Striim.utils import disconnect_signal, delete_attributes
		disconnect_signal(self.__signal_id_1, self.__show_trigger)
		disconnect_signal(self.__signal_id_2, self.__hide_trigger)
		disconnect_signal(self.__signal_id_3, self.__toggle_trigger)
		disconnect_signal(self.__signal_id_4, self)
		disconnect_signal(self.__signal_id_5, self.__close_trigger)
		delete_attributes(self)
		del self
		self = None
		return

	def destroy(self):
		"""
		Destroy this object.

		@param self: Reference to the Trigger instance.
		@type self: A Trigger object.
		"""
		self.__destroy_cb(self)
		return
